//=============================================================================
// (c) Copyright 2011 Diamond Systems Corporation. Use of this source code
// is subject to the terms of Diamond Systems' Software License Agreement.
// Diamond Systems provides no warranty of proper performance if this
// source code is modified.
//
// Module Name: RxTest.c - Test SJA1000 CAN Controller in Receive Mode
//
// Environment: User mode
//    
// Depedency :	DDK Library installed.
//				
//				
// Description: This sample code demonstrates the usage of  the JanusMM CAN driver
//				on Windows XP OS. This code provides the usage of the driver in
//				receive mode to read data coming from the bus.
//				The sample code demonstrates how to Open the device and set the
//				parameters like Baud rate,base address and read from the hardware.
//=============================================================================

#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>

#pragma warning(disable:4201)  // disable nameless struct/union warnings
#include <winioctl.h>
#include <setupapi.h>
#pragma warning(default:4201)

#include "public.h"

#define BUFSIZE 20//(1024 * 11)

static
ULONG   bufsize = BUFSIZE;

#if defined(BUILT_IN_DDK)

#define scanf_s scanf
#define sscanf_s sscanf

#endif

HANDLE
OpenDevice(
    IN CONST GUID * InterfaceGuid,
    IN ULONG        FileFlagOptions
    )
;

void print_usage();

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
int
__cdecl
main(
    __in int argc,
    __in_ecount(argc) char* argv[]
    )
{
    HANDLE      hISAdevice;
    PDWORD      inbuf;
    ULONG       count;
    DWORD       numread;
    ULONG       offset;
    ULONG       length;
    DWORD       numwritten,j=0,dwError=0,Mem_Addr = 0;
    BOOL        okay;
    int       i,k;
    BOOL        hasErrors = FALSE;
    PVERSION_INFO inbuf1 = NULL;

	UCHAR		rxCount,x;

    FILE *fd = NULL, *fp = NULL;
    DWORD err;
    int c,ret,fileXfr = 0;

	int loopcount = 1, infinite = 0, index=0;
	int s, opt, dlc = 0, rtr = 0, extended = 0;
	int verbose = 0,pattern = 0;
	char *data[8];
	char *inputFile = "input.txt",filename[50];
	char *can_name;
	long bit_rate, identifier;
	long address = 0xD7000;

	int dataSize = 0;
	long fileCount = 0;
	char * pEnd;


	if(argc < 2)
	{
		print_usage();
		return 0;

	}

	can_name = argv[1];

	/// parsing the options part
	for( i = 2; i < argc; i++ )
	{
		if( argv[i][0] == '-' )
		{
			switch( argv[i][1] )
			{
				case 'a':		//	bit rate
				{
					address = strtol (argv[++i],&pEnd,16);

					break;
				}
				case 'b':		//	Loop Count
				{
					bit_rate = atoi(argv[++i]);
					break;
				}
				case 'o' :		// Pattern
				{
					inputFile = argv[++i];
					fileXfr = 1;

					break;
				}
				case 'h' : // help
				{
					printf("Help option selected ...\r\n");
					print_usage();
				}
				default:
				{
					printf("Default is activated ...\r\n");
				}
			}
		}
		else
		{
			data[dataSize] = argv[i];
			dataSize++;
		}
	}

	if( ( ret = strcmp(can_name,"canA") ) == 0) {

		hISAdevice = OpenDevice(&GUID_DEVINTERFACE_CAN,
								FILE_FLAG_OVERLAPPED );

		if (hISAdevice == INVALID_HANDLE_VALUE) {
			puts("(ISA) Can't open S5933DK1 device");
			exit(1);
		}
	}
	else {
		hISAdevice = OpenDevice(&GUID_DEVINTERFACE_SJA1000,
								FILE_FLAG_OVERLAPPED );

		if (hISAdevice == INVALID_HANDLE_VALUE) {
			puts("(ISA) Can't open S5933DK1 device");
			exit(1);
		}
	}

    inbuf = (PDWORD) malloc(bufsize);
    if (inbuf == NULL) {
        puts("memory allocation for input buffer failed\n");
        exit(1);
    }

    okay = DeviceIoControl( hISAdevice,
							IOCTL_SET_BAUD_RATE,
							&bit_rate,
							sizeof(bit_rate),
							inbuf,
							bufsize,
							&numread,
							NULL );

	if(!okay)
		printf("CAN Controller baud rate Failed ...\r\n");

    okay = DeviceIoControl( hISAdevice,
							IOCTL_SET_MEM,
							&address,
							sizeof(address),
							inbuf,
							bufsize,
							&numread,
							NULL );

	if(!okay)
		printf("CAN Controller Initialization Failed ...\r\n");


	inbuf1 = malloc(sizeof(VERSION_INFO));

	if(inbuf1 == NULL)
		printf("Out Buffer is NULL ....\r\n");

	if(fileXfr == 1)
	{

		printf("Rx loop count = %ld\r\n",fileCount);

		sprintf(filename,"%ld%s",fileCount++,inputFile);
		printf("file name = %s\r\n",filename);

		fp = fopen(filename,"w");

		if( fp == NULL )
		{
			printf("ERROR :: Opening File gives Error \r\n");
		}

		// READ FROM THE CAN PORT INFINITELY...
		while(1)
		{

			memset(inbuf1, 0, sizeof(VERSION_INFO));

			bufsize=sizeof(VERSION_INFO);

			okay = DeviceIoControl( hISAdevice,
									IOCTL_READ,
									NULL,
									0,
									inbuf1,
									bufsize,
									&numread,
									NULL );

			if(okay){

				rxCount = inbuf1->fi & 0xF;

				for(x = 0; x < rxCount; x++)
				{

					if(inbuf1->rData[x] == 0xFF)
					{
						fclose(fp);

						printf("Rx loop count = %ld\r\n",fileCount);

						sprintf(filename,"%ld%s",fileCount++,inputFile);

						fp = fopen(filename,"w");

						if( fp == NULL )
						{
							printf("ERROR :: Opening File gives Error \r\n");
						}
					}
					else
					{
						fputc((int)inbuf1->rData[x],fp);
					}
				}

			}

		}
	}
	else
	{


			memset(inbuf1, 0, sizeof(VERSION_INFO));

			bufsize=sizeof(VERSION_INFO);

		while(1)
		{

			okay = DeviceIoControl( hISAdevice,
									IOCTL_READ,
									NULL,
									0,
									inbuf1,
									bufsize,
									&numread,
									NULL );

			if(okay && numread){

				if(inbuf1->fi & 0x80)
					printf("[E]   ");
				else
					printf("[S]   ");

				printf("[ID %d]  ",inbuf1->ilVer);

				rxCount = inbuf1->fi & 0xF;

				for(x = 0; x < rxCount; x++)
				{
					printf("%3d ",inbuf1->rData[x]);

				}

				printf("\r\n");

			}

		}


	}

    free(inbuf);
    free(inbuf1);
    CloseHandle(hISAdevice);

    exit((hasErrors)?1:0);
}


HANDLE
OpenDevice(
    IN CONST GUID * InterfaceGuid,
    IN ULONG        FileFlagOptions
    )
{
    SP_DEVICE_INTERFACE_DATA DeviceInterfaceData;
    SP_DEVINFO_DATA DeviceInfoData;
    HDEVINFO hDevInfo;
    PSP_DEVICE_INTERFACE_DETAIL_DATA pDeviceInterfaceDetail = NULL;
    HANDLE hDevice;

    ULONG size;
    int count, i, index;
    BOOL status = TRUE;
    TCHAR *DeviceName = NULL;
    TCHAR *DeviceLocation = NULL;


    //
    //  Retreive the device information for all PLX devices.
    //
    hDevInfo =     SetupDiGetClassDevs(InterfaceGuid,
                                                NULL,
                                                NULL,
                                                DIGCF_DEVICEINTERFACE |
                                                DIGCF_PRESENT );
    if(INVALID_HANDLE_VALUE == hDevInfo) {
        printf("No AMCC devices are present and enabled in the system\n");
        return INVALID_HANDLE_VALUE;
    }
    //
    //  Initialize the SP_DEVICE_INTERFACE_DATA Structure.
    //
    DeviceInterfaceData.cbSize  = sizeof(SP_DEVICE_INTERFACE_DATA);

    //
    //  Determine how many devices are present.
    //
    count = 0;
    while(SetupDiEnumDeviceInterfaces(hDevInfo,
                                    NULL,
                                    InterfaceGuid,
                                    count++,  //Cycle through the available devices.
                                    &DeviceInterfaceData));

    //
    // Since the last call fails when all devices have been enumerated,
    // decrement the count to get the true device count.
    //
    count--;

    //
    //  If the count is zero then there are no devices present.
    //
    if(count == 0)
    {
        printf("No AMCC devices are present and enabled in the system.\n");
        goto End;
    }


    //
    //  Initialize the appropriate data structures in preparation for
    //  the SetupDi calls.
    //
    DeviceInterfaceData.cbSize = sizeof(SP_DEVICE_INTERFACE_DATA);
    DeviceInfoData.cbSize = sizeof(SP_DEVINFO_DATA);


    //
    //  Loop through the device list to allow user to choose
    //  a device.  If there is only one device, select it
    //  by default.
    //
    i = 0;
    while(SetupDiEnumDeviceInterfaces(hDevInfo,
                                NULL,
                                (LPGUID)InterfaceGuid,
                                i,
                                &DeviceInterfaceData))
    {

        //
        // Determine the size required for the DeviceInterfaceData
        //
        SetupDiGetDeviceInterfaceDetail(hDevInfo,
                                        &DeviceInterfaceData,
                                        NULL,
                                        0,
                                        &size,
                                        NULL);

        if(GetLastError() != ERROR_INSUFFICIENT_BUFFER)
        {
            printf("SetupDiGetDeviceInterfaceDetail failed, Error: %d", GetLastError());
            goto End;
        }
        pDeviceInterfaceDetail = (PSP_DEVICE_INTERFACE_DETAIL_DATA)malloc(size);
        if(!pDeviceInterfaceDetail)
        {
            printf("Insufficient memory.\n");
            goto End;
        }

        //
        // Initialize structure and retrieve data.
        //
        pDeviceInterfaceDetail->cbSize = sizeof(SP_DEVICE_INTERFACE_DETAIL_DATA);
        status = SetupDiGetDeviceInterfaceDetail(hDevInfo,
                                    &DeviceInterfaceData,
                                    pDeviceInterfaceDetail,
                                    size,
                                    NULL,
                                    &DeviceInfoData);

        free(pDeviceInterfaceDetail);
        pDeviceInterfaceDetail = NULL;

        if(!status)
        {
            printf("SetupDiGetDeviceInterfaceDetail failed, Error: %d", GetLastError());
            goto End;
        }

        //
        //  Get the Device Name
        //  Calls to SetupDiGetDeviceRegistryProperty require two consecutive
        //  calls, first to get required buffer size and second to get
        //  the data.
        //
        SetupDiGetDeviceRegistryProperty(hDevInfo,
                                        &DeviceInfoData,
                                        SPDRP_DEVICEDESC,
                                        NULL,
                                        (PBYTE)DeviceName,
                                        0,
                                        &size);

        if(GetLastError() != ERROR_INSUFFICIENT_BUFFER)
        {
            printf("SetupDiGetDeviceRegistryProperty failed, Error: %d", GetLastError());
            goto End;
        }

        DeviceName = (TCHAR*) malloc(size);
        if(!DeviceName)
        {
            printf("Insufficient memory.\n");
            goto End;
        }

        status = SetupDiGetDeviceRegistryProperty(hDevInfo,
                                        &DeviceInfoData,
                                        SPDRP_DEVICEDESC,
                                        NULL,
                                        (PBYTE)DeviceName,
                                        size,
                                        NULL);
        if(!status)
        {
            printf("SetupDiGetDeviceRegistryProperty failed, Error: %d", GetLastError());
            free(DeviceName);
            goto End;
        }

        //
        //  Now retrieve the Device Location.
        //
        SetupDiGetDeviceRegistryProperty(hDevInfo,
                                        &DeviceInfoData,
                                        SPDRP_LOCATION_INFORMATION,
                                        NULL,
                                        (PBYTE)DeviceLocation,
                                        0,
                                        &size);

        if(GetLastError() == ERROR_INSUFFICIENT_BUFFER)
        {
            DeviceLocation = (TCHAR*) malloc(size);

            if (DeviceLocation != NULL)
            {
                status = SetupDiGetDeviceRegistryProperty(hDevInfo,
                                                          &DeviceInfoData,
                                                          SPDRP_LOCATION_INFORMATION,
                                                          NULL,
                                                          (PBYTE)DeviceLocation,
                                                          size,
                                                          NULL);
                if(!status)
                {
                    free(DeviceLocation);
                    DeviceLocation = NULL;
                }
            }

        } else {
            DeviceLocation = NULL;
        }

        //
        // If there is more than one device print description.
        //
        if(count > 1)
        {
            printf("%d- ", i);
        }

        printf("%s\n", DeviceName);

        if(DeviceLocation)
        {
            printf("        %s\n", DeviceLocation);
        }

        free(DeviceName);
        free(DeviceLocation);

        i++; // Cycle through the available devices.
    }

    //
    //  Select device.
    //
    index = 0;
    if(count > 1)
    {
        printf("\nSelect Device: ");
        (void)scanf_s("%d", &index);
    }

    //
    //  Get information for specific device.
    //

    status = SetupDiEnumDeviceInterfaces(hDevInfo,
                                    NULL,
                                    (LPGUID)InterfaceGuid,
                                    index,
                                    &DeviceInterfaceData);

    if(!status)
    {
        printf("SetupDiEnumDeviceInterfaces failed, Error: %d", GetLastError());
        goto End;
    }
    //
    // Determine the size required for the DeviceInterfaceData
    //
    SetupDiGetDeviceInterfaceDetail(hDevInfo,
                                    &DeviceInterfaceData,
                                    NULL,
                                    0,
                                    &size,
                                    NULL);

    if(GetLastError() != ERROR_INSUFFICIENT_BUFFER)
    {
        printf("SetupDiGetDeviceInterfaceDetail failed, Error: %d", GetLastError());
        goto End;
    }
    pDeviceInterfaceDetail = (PSP_DEVICE_INTERFACE_DETAIL_DATA)malloc(size);
    if(!pDeviceInterfaceDetail)
    {
        printf("Insufficient memory.\n");
        goto End;
    }

    //
    // Initialize structure and retrieve data.
    //
    pDeviceInterfaceDetail->cbSize = sizeof(SP_DEVICE_INTERFACE_DETAIL_DATA);
    status = SetupDiGetDeviceInterfaceDetail(hDevInfo,
                                &DeviceInterfaceData,
                                pDeviceInterfaceDetail,
                                size,
                                NULL,
                                &DeviceInfoData);
    if(!status)
    {
        printf("SetupDiGetDeviceInterfaceDetail failed, Error: %d", GetLastError());
        goto End;
    }

    //
    //  Get handle to device.
    //
    hDevice = CreateFile(pDeviceInterfaceDetail->DevicePath,
                            GENERIC_READ|GENERIC_WRITE,
                            FILE_SHARE_READ | FILE_SHARE_WRITE,
                            NULL,
                            OPEN_EXISTING,
                            FileFlagOptions,
                            NULL
                            );

    if(hDevice == INVALID_HANDLE_VALUE)
    {
        printf("CreateFile failed.  Error:%d", GetLastError());
    }

    free(pDeviceInterfaceDetail);
    SetupDiDestroyDeviceInfoList(hDevInfo);

    return hDevice;

End:
    if(pDeviceInterfaceDetail) {
        free(pDeviceInterfaceDetail);
    }
    SetupDiDestroyDeviceInfoList(hDevInfo);
    return INVALID_HANDLE_VALUE;
}


void print_usage()
{

	printf(
		"Usage: canRecv.exe [<can-interface>] [Options] <can-msg>\n"
		"Options:\n"
		" -b    [ ex : -b br_value ] baud-rate (default = 1000000) \n"
		" -a    [ ex : -a address  ] address (default = 0xD7000) \n"
		" -o    <filename>\t\t"    "output filename\n"
		" -h,   --help		this help\n"
		);

}